unit Sample15;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim, ComCtrls;

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    ProgressBar: TProgressBar;
    Tabulator0: TTabulator;
    Tabulator1: TTabulator;
    MultiRand1: TMultiRand;
    MultiRand2: TMultiRand;
    RegularOrders: TGenerator;
    UrgentOrders: TGenerator;
    Queue: TQueuePrty;
    Processing: TServer;
    Finishing: TTerminator;
    Model: TModel;
    procedure Button1Click(Sender: TObject);
    procedure RegularOrdersBeforeTimeGoOn(Sender: TAggregate);
    procedure RegularOrdersExit(Sender: TBlock; Trans: TTransaction);
    procedure RegularOrdersRouting(Sender: TBlock; Trans: TTransaction);
    procedure UrgentOrdersAfterGeneration(Sender: TBlock;
      Trans: TTransaction);
    procedure UrgentOrdersBeforeTimeGoOn(Sender: TAggregate);
    procedure UrgentOrdersExit(Sender: TBlock; Trans: TTransaction);
    procedure UrgentOrdersRouting(Sender: TBlock; Trans: TTransaction);
    procedure QueueRouting(Sender: TBlock; Trans: TTransaction);
    procedure ProcessingEnter(Sender: TBlock; Trans: TTransaction);
    procedure ProcessingPreempt(Sender: TBlock; Trans: TTransaction);
    procedure ProcessingRouting(Sender: TBlock; Trans: TTransaction);
    procedure FinishingEnter(Sender: TBlock; Trans: TTransaction);
    procedure ModelNewTime(Sender: TAggregate; Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  MyTransaction = class(TTransaction)
  public
     BirthTime: real;
  end;

var
  Form1: TForm1;
  LimitTime: real;           // Simulation time
  QCapacity: longint;        // Queue capacity

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
begin
   LimitTime:=264.0;
   Memo.Lines.Clear;
   Model.SetTransactionClass(MyTransaction);
   Model.Simulate(LimitTime);
   Memo.Lines.Add('1. Regular orders');
   Memo.Lines.Add('   Number of arrived orders: '+IntToStr(RegularOrders.Exits));
   Memo.Lines.Add('   Number of executed orders: '+IntToStr(Tabulator0.Count));
   Memo.Lines.Add('   The average execution time: '+FormatFloat('0.00',
                                                    Tabulator0.Mean));

   Memo.Lines.Add('   The deviation of execution time: '+FormatFloat('0.00',
                                                         Tabulator0.Deviation));
   Memo.Lines.Add('2. Urgent orders');
   Memo.Lines.Add('   Number of arrived orders: '+IntToStr(UrgentOrders.Exits));
   Memo.Lines.Add('   Number of executed orders: '+IntToStr(Tabulator1.Count));
   Memo.Lines.Add('   The average execution time: '+FormatFloat('0.00',
                                                     Tabulator1.Mean));

   Memo.Lines.Add('   The deviation of execution time: '+FormatFloat('0.00',
                                                         Tabulator1.Deviation));
   Memo.Lines.Add('__________________________________________________');
   Memo.Lines.Add(' The average number of orders on processing: '+
                                                     FormatFloat('0.00',
                                                     Processing.AverageCount));
   Memo.Lines.Add(' The maximal number of orders on processing: '+
                                                     IntToStr(Processing.MaxCount));
   Memo.Lines.Add(' The loading of firm: '+FormatFloat('0.00',Processing.Usage));
   Memo.Lines.Add(' The average length of the queue: '+
                                           FormatFloat('0.00',Queue.AverageCount));
   Memo.Lines.Add(' The maximal length of the queue: '+
                                                     IntToStr(Queue.MaxCount));
      Memo.Lines.Add(' The cost of executed orders: '+FormatFloat('0.00',
                                 Tabulator0.Count*10000.0+Tabulator1.Count*20000.0));
   MultiRand.Reset;

   Model.Reset;
end;

procedure TForm1.RegularOrdersBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.RegularOrdersExit(Sender: TBlock; Trans: TTransaction);
begin
  (Trans as MyTransaction).BirthTime:=ModelTime;
  Sender.NextTime(MultiRand.Exponential(5.0));
end;

procedure TForm1.RegularOrdersRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Queue);
end;

procedure TForm1.UrgentOrdersAfterGeneration(Sender: TBlock;
  Trans: TTransaction);
begin
  Trans.SetPrty(1);
  Trans.SetPreempt;
end;

procedure TForm1.UrgentOrdersBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.UrgentOrdersExit(Sender: TBlock; Trans: TTransaction);
begin
   (Trans as MyTransaction).BirthTime:=ModelTime;
   Sender.NextTime(MultiRand1.Exponential(15.0));
end;

procedure TForm1.UrgentOrdersRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Queue);
end;

procedure TForm1.QueueRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Processing);
end;

procedure TForm1.ProcessingEnter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand2.Exponential(3.5));
end;

procedure TForm1.ProcessingPreempt(Sender: TBlock; Trans: TTransaction);
begin
  Sender.Postpone;
end;

procedure TForm1.ProcessingRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Finishing);
end;

procedure TForm1.FinishingEnter(Sender: TBlock; Trans: TTransaction);
begin
   if Trans.GetPrty=0 then
      Tabulator0.PutValue(ModelTime-(Trans as MyTransaction).BirthTime)
   else
      Tabulator1.PutValue(ModelTime-(Trans as MyTransaction).BirthTime);
end;


procedure TForm1.ModelNewTime(Sender: TAggregate; Trans: TTransaction);
var  NewPos: integer;
     Ratio: real;
begin
   Ratio:=ModelTime/LimitTime;
   if Ratio>1.0 then Ratio:=1.0;
   NewPos:=Round(Ratio*100.0);
   with ProgressBar do
      if Position<>NewPos then Position:=NewPos;
end;

end.

