unit Sample5;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim;

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    Entrance: TGenerator;
    Hall: TQueue;
    Barber: TServer;
    ExitDoor: TTerminator;
    Model: TModel;
    procedure Button1Click(Sender: TObject);
    procedure EntranceBeforeTimeGoOn(Sender: TAggregate);
    procedure EntranceExit(Sender: TBlock; Trans: TTransaction);
    procedure EntranceRouting(Sender: TBlock; Trans: TTransaction);
    procedure HallRouting(Sender: TBlock; Trans: TTransaction);
    procedure BarberEnter(Sender: TBlock; Trans: TTransaction);
    procedure BarberRouting(Sender: TBlock; Trans: TTransaction);
    procedure ModelAfterPass(Sender, Receiver: TBlock;
      Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


function PadCh(S : ShortString; Ch : Char; Len : Byte) : string;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
begin
   Model.Simulate(480.0);
end;

procedure TForm1.EntranceBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.EntranceExit(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Uniform(12.0,24.0));
end;

procedure TForm1.EntranceRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Hall);
end;

procedure TForm1.HallRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Barber);
end;

procedure TForm1.BarberEnter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Uniform(14.0,20.0));
end;

procedure TForm1.BarberRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(ExitDoor);
end;

procedure TForm1.ModelAfterPass(Sender, Receiver: TBlock;
  Trans: TTransaction);
begin
  Memo.Lines.Add(PadCh(Sender.Name,' ',12)+                  // Name of Sender
                 PadCh(Receiver.Name,' ',12)+                // Name of Receiver
                 PadCh(IntToStr(Trans.GetTransID),' ',6)+    // Transaction ID
                 FormatFloat('0.00',ModelTime));             // System time
end;


{Return a string right-padded to length len with ch}
function PadCh(S : ShortString; Ch : Char; Len : Byte) : string;
var
   o : ShortString;
begin
   S:=Trim(S);
   if Length(S) >= Len then
     Result := S
   else
   begin
     SetLength(o,Len);
     Move(S[1], o[1], Length(S));
     FillChar(o[Succ(Length(S))], Len-Length(S), Ch);
     Result := o;
   end;
end;



end.
