unit Sample7;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim, ComCtrls;

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    ProgressBar: TProgressBar;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    EditSim: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    EditCap: TEdit;
    Entrance: TGenerator;
    Hall: TQueue;
    Barber: TServer;
    ExitDoor: TTerminator;
    AnotherShop: TTerminator;
    Model: TModel;
    procedure Button1Click(Sender: TObject);
    procedure EditSimExit(Sender: TObject);
    procedure EditCapExit(Sender: TObject);
    procedure EntranceBeforeTimeGoOn(Sender: TAggregate);
    procedure EntranceExit(Sender: TBlock; Trans: TTransaction);
    procedure EntranceRouting(Sender: TBlock; Trans: TTransaction);
    procedure HallEnter(Sender: TBlock; Trans: TTransaction);
    procedure HallRouting(Sender: TBlock; Trans: TTransaction);
    procedure HallTimeFinish(Sender: TBlock; Trans: TTransaction);
    procedure BarberEnter(Sender: TBlock; Trans: TTransaction);
    procedure BarberRouting(Sender: TBlock; Trans: TTransaction);
    procedure ModelNewTime(Sender: TAggregate; Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;



var
  Form1: TForm1;
  LimitTime: real;           // Simulation time
  QCapacity: longint;        // Queue capacity

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
begin
   try
      LimitTime:=StrToFloat(Trim(EditSim.Text));
   except
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;
   if LimitTime<=0.0 then
   begin
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;

   try
      QCapacity:=StrToInt(Trim(EditCap.Text));
   except
      MessageDlg('Invalid capacity',mtError,[mbOK],0);
      EditCap.SetFocus;
      Exit;
   end;
   if QCapacity<0 then
   begin
      MessageDlg('Invalid capacity',mtError,[mbOK],0);
      EditCap.SetFocus;
      Exit;
   end;

   Hall.Capacity:=QCapacity;
   Memo.Lines.Clear;
   Model.Simulate(LimitTime);
   Memo.Lines.Add('Capacity: '+EditCap.Text);
   Memo.Lines.Add('Total customers: '+IntToStr(Entrance.Exits));
   Memo.Lines.Add('Total amount of lost customers:  '+
                  IntToStr(AnotherShop.Entries));
   Memo.Lines.Add('Lost customers due to limited time waiting:  '+
                  IntToStr(Hall.TimeLimitExits));
   Memo.Lines.Add('Lost customers due to limited capacity:  '+
                  IntToStr(AnotherShop.Entries-Hall.TimeLimitExits));
   Memo.Lines.Add('Percent of lost customers: '+FormatFloat('0.00',
                  AnotherShop.Entries*100.0/Entrance.Exits)+'%');
   MultiRand.Reset;
   Model.Reset;
end;

procedure TForm1.EditSimExit(Sender: TObject);
begin
   try
      LimitTime:=StrToFloat(Trim(EditSim.Text));
   except
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;
   if LimitTime<=0.0 then
   begin
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
   end;
end;

procedure TForm1.EditCapExit(Sender: TObject);
begin
   try
      QCapacity:=StrToInt(Trim(EditCap.Text));
   except
      MessageDlg('Invalid capacity',mtError,[mbOK],0);
      EditCap.SetFocus;
      Exit;
   end;
   if QCapacity<0 then
   begin
      MessageDlg('Invalid capacity',mtError,[mbOK],0);
      EditCap.SetFocus;
      Exit;
   end;
   if QCapacity=0 then
      MessageDlg('Unlimited capacity',mtInformation,[mbOK],0);
   Hall.Capacity:=QCapacity;
end;


procedure TForm1.EntranceBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.EntranceExit(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(11.0));
end;

procedure TForm1.EntranceRouting(Sender: TBlock; Trans: TTransaction);
begin
  if Hall.isReadyToReceive(Trans) then
     Sender.PassTo(Hall)
  else
     Sender.PassTo(AnotherShop);
end;


procedure TForm1.HallEnter(Sender: TBlock; Trans: TTransaction);
begin
  if MultiRand.Uniform(0.0,1.0) > 0.5 then    // only half of customers can't wait 
     Sender.NextTime(MultiRand.Uniform(20.0,30.0));
end;

procedure TForm1.HallRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Barber);
end;

procedure TForm1.HallTimeFinish(Sender: TBlock; Trans: TTransaction);
begin
   Sender.PassTo(AnotherShop);          // Pass to terminator
end;

procedure TForm1.BarberEnter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(10.0));
end;

procedure TForm1.BarberRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(ExitDoor);
end;

procedure TForm1.ModelNewTime(Sender: TAggregate; Trans: TTransaction);
var  NewPos: integer;
     Ratio: real;
begin
   Ratio:=ModelTime/LimitTime;
   if Ratio>1.0 then Ratio:=1.0;
   NewPos:=Round(Ratio*100.0);
   with ProgressBar do
      if Position<>NewPos then Position:=NewPos;
end;

end.


